/*
                              SNR measure Script

   A script for automating SMR measuring of input image files.

*/

#feature-id    Batch Processing > SNR measure

#feature-info  An automated SNR weight measuring utility.<br/>\
   <br/> \
   A script designed to calculate integration weight based on SNR for input images and\
   store calculated value as SNRW FITS keyword.\

#feature-icon  SNRmeasure.xpm

#include <pjsr/NumericControl.jsh>
#include <pjsr/Sizer.jsh>
#include <pjsr/DataType.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/StdButton.jsh>
#include <pjsr/StdIcon.jsh>
#include <pjsr/StdCursor.jsh>
#include <pjsr/UndoFlag.jsh>

#define VERSION "1.0.0"
#define TITLE   "SNR measure"

//label object constructor
function labelBox(parent, elText, tAlign, elWidth)
{
   this.label = new Label(parent);
   this.label.text = elText;
   this.label.textAlignment = tAlign;
   this.label.setMaxWidth(elWidth);

   return this.label;
}

//push button object constructor
function pushButton(parent, bText, bIcon, bToolTip)
{
   this.button = new PushButton(parent);
   if(bText != '')
      this.button.text = bText;
   if(bIcon != '')
      this.button.icon = new Bitmap(bIcon);
   if(bToolTip != '')
      this.button.toolTip = bToolTip;

   return this.button;
}

//tool button object constructor
function toolButton(parent, tbIcon, tbToolTip)
{
   this.toolButton = new ToolButton(parent);
   if(tbIcon != '')
      this.toolButton.icon = new Bitmap(tbIcon);
   if(tbToolTip != '')
      this.toolButton.toolTip = tbToolTip;

      return this.toolButton;
}

//edit box object constructor
function editBox(parent, eText, readWrite, eStyle, eMinWidth)
{
   this.edit = new Edit(parent);
   if(eText != '')
      this.edit.text = eText;
   this.edit.readOnly = readWrite;
   this.edit.style = eStyle;
   this.edit.setMinWidth(eMinWidth);

   return this.edit;
}

//spin box object constructor
function spinBox(parent, sbValue, sbMinVal, sbMaxVal, sbStep, sbCanEdit)
{
   this.spinbox = new SpinBox(parent);
   this.spinbox.value = sbValue;
   this.spinbox.setRange(sbMinVal, sbMaxVal);
   this.spinbox.stepSize = sbStep;
   this.spinbox.editable = sbCanEdit;

   return this.spinbox;
}

//combobox object constructor
function comboBox(parent, bToolTip)
{

   this.comboBox = new ComboBox(parent);
   this.comboBox.editEnabled = false;
   this.comboBox.setScaledFixedWidth(500);
   this.comboBox.toolTip = bToolTip;

   return this.comboBox;
}

//numeric control object constructor
function numericControl(parent, ncVal, ncPrec, ncLow, ncHigh, slLow, slHigh, minW, lText, tTip)
{
   this.nc = new NumericControl(parent);
   this.nc.setValue(ncVal);
   this.nc.setPrecision(ncPrec);
   this.nc.setRange(ncLow, ncHigh);
   this.nc.slider.setRange(slLow, slHigh);
   this.nc.slider.scaledMinWidth = minW;
   if(lText.length != 0)
      this.nc.label.text = lText;
   if(tTip.length != 0)
      this.nc.toolTip = tTip;

   return this.nc;
}

//function to parse time given in seconds
var tTime = new Array(0,0,0);
function Time()
{
   this.parseTime = function(sec)
   {
      var res = 0;
      tTime[0] = 0;
      tTime[1] = 0;
      tTime[2] = 0;

      if(sec >= 3600)
      {
         res = Math.floor(sec/3600);
         tTime[0] = res;
         sec = sec - (3600*res);
      }

      if(sec >= 60)
      {
         res = Math.floor(sec/60);
         tTime[1] = res;
         sec = sec - (60*res);
      }

      tTime[2] =sec;
   }
}
var myTime = new Time();

function GlobalData()
{
   this.inputFiles = new Array;
   this.signal = new Array;
   this.back = new Array;
   this.snr = new Array;
   this.ns = new Array;
   this.ipCount = 0;

   var rect = ImageWindow.activeWindow.previews[0].window.previewRect( ImageWindow.activeWindow.previews[0] );
   this.back_x1 = rect.x0;
   this.back_y1 = rect.y0;
   this.back_x2 = rect.x0 + rect.width;
   this.back_y2 = rect.y0 + rect.height;
   this.obj_x1 = rect.x0;
   this.obj_y1 = rect.y0;
   this.obj_x2 = rect.x0 + rect.width;
   this.obj_y2 = rect.y0 + rect.height;
   this.back_selected = false;
   this.obj_selected = false;
}


var data = new GlobalData();  //variable for global access to script data

//script engine
function ChannelExtractionEngine()
{
   this.execute = function()
   {
      var errors = 0;
      var pStart = Date.now();

      console.writeln("<br>################=============================################");
      console.writeln("################ <b>SNR weight measuring script</b> ################");
      console.writeln("################=============================################<br>");
      console.writeln("Processing ", data.ipCount, " image files...");

      console.writeln(format("Target background coordinates: (%d, %d) - (%d, %d)", data.back_x1, data.back_y1, data.back_x2, data.back_y2));
      console.writeln(format("Target object coordinates: (%d, %d) - (%d, %d)", data.obj_x1, data.obj_y1, data.obj_x2, data.obj_y2));

      var back_window = new ImageWindow (
         data.back_x2 - data.back_x1,
         data.back_y2 - data.back_y1,
         1,
         32,
         true,
         false,
         "Background"
      );
      back_window.mainView.beginProcess( UndoFlag_NoSwapFile );

      var obj_window = new ImageWindow (
         data.obj_x2 - data.obj_x1,
         data.obj_y2 - data.obj_y1,
         1,
         32,
         true,
         false,
         "Object"
      );
      obj_window.mainView.beginProcess( UndoFlag_NoSwapFile );

      for (var i = 0, n = data.ipCount; i < n; ++i)
      {
         console.writeln("<br>File #", (i+1));

         var srcWin = ImageWindow.open(data.inputFiles[i]);

         var back_pixels = [];
         var back_rect_src = new Rect(data.back_x1, data.back_y1, data.back_x2, data.back_y2);
         var back_rect_trg = new Rect(0, 0, data.back_x2 - data.back_x1, data.back_y2 - data.back_y1);
         srcWin[0].mainView.image.getSamples(back_pixels, back_rect_src, 0);
         back_window.mainView.image.setSamples(back_pixels, back_rect_trg, 0);

         var obj_pixels = [];
         var obj_rect_src = new Rect(data.obj_x1, data.obj_y1, data.obj_x2, data.obj_y2);
         var obj_rect_trg = new Rect(0, 0, data.obj_x2 - data.obj_x1, data.obj_y2 - data.obj_y1);
         srcWin[0].mainView.image.getSamples(obj_pixels, obj_rect_src, 0);
         obj_window.mainView.image.setSamples(obj_pixels, obj_rect_trg, 0);

         var back_mean = back_window.mainView.image.mean();
         var back_stdDev = back_window.mainView.image.stdDev();
         var obj_mean = obj_window.mainView.image.mean();

         var snr = (obj_mean - back_mean) / back_stdDev;

         data.signal[i] = obj_mean - back_mean;
         data.back[i] = back_mean;
         data.snr[i] = snr;
         data.ns[i] = back_stdDev;

         console.writeln("SNR = ", format("%.3e", snr).replace("e", "E"));
         console.writeln("Noise = ", format("%.6e", back_stdDev).replace("e", "E"));

         var keywords = srcWin[0].keywords;
         for (var k in keywords) {
	    if (keywords[k].name == 'SNRW') {
               keywords.splice(k, 1);
            }
         }

         srcWin[0].keywords = keywords.concat([
            new FITSKeyword(
              "SNRW",
              format("%.4e", snr).replace("e", "E"),
              "Image SNR weight"
            )
         ]);

         srcWin[0].save();
         srcWin[0].forceClose();
      }

      back_window.mainView.endProcess;
      back_window.forceClose();
      obj_window.mainView.endProcess;
      obj_window.forceClose();
      var pStop = Date.now();
      var dTime = (pStop-pStart)/1000;
      myTime.parseTime(dTime);
      var copStr = format("%i hours   %i mins   %.3f secs", tTime[0], tTime[1], tTime[2]);
      console.writeln("<br>SNR estimate complete");
      console.writeln("<br>Process time: ", copStr);
   }
}
var engine = new ChannelExtractionEngine();

//////////// Main Dialog %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function mainDialog()
{
   this.__base__ = Dialog;
   this.__base__();

   var ttStr = "";

   //script info label
   this.helpLabel = new Label(this);
   with(this.helpLabel)
   {
      frameStyle = FrameStyle_Box;
      margin = 4;
      wordWrapping = true;
      useRichText = true;
      text = "<p><b>" + TITLE + " v" + VERSION + "</b> &mdash; Calculates integration weight based on SNR for input images and store calculated value as <b>SNRW</b> FITS keyword</p>";
   }

   //input controls
   this.inputFiles_TreeBox = new TreeBox(this);
   with(this.inputFiles_TreeBox)
   {
      alternateRowColor = true;
      multipleSelection = false;
      headerVisible = false;
      numberOfColumns = 3;
      showColumn(2, false);
      rootDecoration = false;
      setScaledMinSize(300, 200);
      setColumnWidth(0, this.logicalPixelsToPhysical( 20 ));
      setColumnWidth(1, this.logicalPixelsToPhysical( 300 ));
   }

   this.addFiles_PushButton = new pushButton(this, "Add files", "", "Add files to the input list.");
   this.addFiles_PushButton.onClick = function()
   {
      var ofd = new OpenFileDialog;
      ofd.multipleSelections = true;
      ofd.caption = "Select Images";
      ofd.loadImageFilters();

      if (ofd.execute())
      {
         this.dialog.inputFiles_TreeBox.clear();
         this.dialog.inputFiles_TreeBox.canUpdate = false;
         for(var i = 0, n = ofd.fileNames.length; i < n; ++i)
         {
            data.inputFiles.push(ofd.fileNames[i]);
            ++data.ipCount;
         }

         data.inputFiles.sort();

         for(var i = 0, n = data.ipCount; i < n; ++i)
         {
            var node = new TreeBoxNode(this.dialog.inputFiles_TreeBox);
            node.checkable = true;
            node.checked = false;
            node.setText(0, (i+1).toString());
            node.setText(1, File.extractName(data.inputFiles[i]));
            node.setText(2, data.inputFiles[i]);
         }
         this.dialog.inputFiles_TreeBox.canUpdate = true;
      }
   }

   this.selectAll_PushButton = new pushButton(this, "Select All", "", "Select all of the listed input files.");
   this.selectAll_PushButton.onClick = function()
   {
      for(var i = 0, n = this.dialog.inputFiles_TreeBox.numberOfChildren; i < n; ++i)
         this.dialog.inputFiles_TreeBox.child(i).checked = true;
   }

   this.invertSelected_PushButton = new pushButton(this, "Invert Selection", "", "Invert the input files selection.");
   this.invertSelected_PushButton.onClick = function()
   {
      for(var i = 0, n = this.dialog.inputFiles_TreeBox.numberOfChildren; i < n; ++i)
         this.dialog.inputFiles_TreeBox.child(i).checked =
            !this.dialog.inputFiles_TreeBox.child(i).checked;
   }

   this.removeSelected_PushButton = new pushButton(this, "Remove Selected", "", "Remove the selected files from the input list.");
   this.removeSelected_PushButton.onClick = function()
   {
      for(var i = this.dialog.inputFiles_TreeBox.numberOfChildren; --i >= 0; )
         if(this.dialog.inputFiles_TreeBox.child(i).checked)
            this.dialog.inputFiles_TreeBox.remove(i);

      data.inputFiles.length = 0;
      data.ipCount = 0;
      for(var i = 0, n = this.dialog.inputFiles_TreeBox.numberOfChildren; i < n; ++i)
      {
         this.dialog.inputFiles_TreeBox.child(i).setText(0, (i+1).toString());
         data.inputFiles.push(this.dialog.inputFiles_TreeBox.child(i).text(2));
         ++data.ipCount;
      }
   }

   this.clear_PushButton = new pushButton(this, "Clear", "", "Remove all of the input files from the list.");
   this.clear_PushButton.onClick = function()
   {
      this.dialog.inputFiles_TreeBox.clear();
      data.inputFiles.length = 0;
      data.ipCount = 0;
   }

   this.inputBtns_Sizer = new VerticalSizer;
   with(this.inputBtns_Sizer)
   {
      margin = 4;
      spacing = 6;
      add(this.addFiles_PushButton);
      add(this.selectAll_PushButton);
      add(this.invertSelected_PushButton);
      add(this.removeSelected_PushButton);
      add(this.clear_PushButton);
      addStretch();
   }

   this.inputControls_GroupBox = new GroupBox(this);
   with(this.inputControls_GroupBox)
   {
      title = "Input Controls";
      sizer = new HorizontalSizer;
      sizer.margin = 4;
      sizer.spacing = 6;
      sizer.add(this.inputFiles_TreeBox);
      sizer.add(this.inputBtns_Sizer);
   }

   this.back_ComboBox = new comboBox(this, "Select preview with background");

   for (var i = 0; i < ImageWindow.activeWindow.previews.length; i++) {
      this.back_ComboBox.addItem(ImageWindow.activeWindow.previews[i].fullId);
   }

   this.back_ComboBox.onItemSelected = function( index )
   {
      var rect = ImageWindow.activeWindow.previews[index].window.previewRect( ImageWindow.activeWindow.previews[index] );
      data.back_x1 = rect.x0;
      data.back_y1 = rect.y0;
      data.back_x2 = rect.x0 + rect.width;
      data.back_y2 = rect.y0 + rect.height;
      data.back_selected = true;
   }


   this.backControls_GroupBox = new GroupBox(this);
   with(this.backControls_GroupBox)
   {
      title = "Background preview";
      sizer = new HorizontalSizer;
      sizer.margin = 4;
      sizer.spacing = 6;
      sizer.add(this.back_ComboBox);
   }

   this.obj_ComboBox = new comboBox(this, "Select preview with object");

   for (var i = 0; i < ImageWindow.activeWindow.previews.length; i++) {
      this.obj_ComboBox.addItem(ImageWindow.activeWindow.previews[i].fullId);
   }

   this.obj_ComboBox.onItemSelected = function( index )
   {
      var rect = ImageWindow.activeWindow.previews[index].window.previewRect( ImageWindow.activeWindow.previews[index] );
      data.obj_x1 = rect.x0;
      data.obj_y1 = rect.y0;
      data.obj_x2 = rect.x0 + rect.width;
      data.obj_y2 = rect.y0 + rect.height;
      data.obj_selected = true;
   }


   this.objControls_GroupBox = new GroupBox(this);
   with(this.objControls_GroupBox)
   {
      title = "Object preview";
      sizer = new HorizontalSizer;
      sizer.margin = 4;
      sizer.spacing = 6;
      sizer.add(this.obj_ComboBox);
   }


   //dialog control buttons
   ttStr = "Run the SNR estimating engine for the selected input files.";
   this.ok_Button = new pushButton(this, "Execute", "", ttStr);
   this.ok_Button.onClick = function()
   {
      this.dialog.ok();
   }

   ttStr = "Close the script.";
   this.cancel_Button = new pushButton(this, "Close", "", ttStr);
   this.cancel_Button.onClick = function()
   {
      this.dialog.cancel();
   }

   //dialog control buttons sizer
   this.buttons_Sizer = new HorizontalSizer;
   with(this.buttons_Sizer)
   {
      spacing = 6;
      addStretch();
      add( this.ok_Button );
      add( this.cancel_Button );
   }

   //main dialog sizers

   this.sizer = new VerticalSizer;
   with(this.sizer)
   {
      margin = 6;
      spacing = 6;
      add( this.helpLabel );
      addSpacing( 4 );
      add(this.inputControls_GroupBox);
      add(this.backControls_GroupBox);
      add(this.objControls_GroupBox);
      add(this.buttons_Sizer);
   }

   this.windowTitle = TITLE + " Script";
   this.adjustToContents();
}

mainDialog.prototype = new Dialog;
var maindlg = new mainDialog();

function main()
{
   console.hide();

   // Show our dialog box, quit if cancelled.
   for ( ;; )
   {
      if (maindlg.execute())
      {
         if(data.inputFiles.length == 0)
         {
            var msgStr = "<p>There are no input files listed.</p>" +
                           "<p>Do you wish to continue?</p>";
            var msg = new MessageBox(msgStr, TITLE, StdIcon_Error, StdButton_Yes, StdButton_No);
            if(msg.execute() == StdButton_Yes)
               continue;
            else
               break;
         }
         else
         {
            engine.execute();
            break;
         }
      }

      break;
   }

   console.hide();
}

main();
